from .librus_session import LibrusSession
from .librus_terminal import LibrusTerminal


def main():
    session = LibrusSession()

    terminal = LibrusTerminal(session)

    terminal.run_terminal()


if __name__ == "__main__":
    main()



from .librus_session import LibrusSession


def help_command(session: LibrusSession) -> None:
    message = """
Librus terminal

Komendy:

    gmes - pobiera, wyświetla i zapisuje w pamięci programu listę wiadomosci w następującym formacie: IndeksWiadomości InformacjeDodatkowe Data Nadawca Temat
        Użycie:
            gmes <arguments>
                arguments:
                    - --source [options] - skąd mają być pobrane wiadomosci, opcje to:
                        - archive - wiadomosci będą pobrane z archiwum
                        - current - wiadomosci z aktualnego roku szkolnego
                        - mamory - wiadomosci zapisane w pamięci programu
                    - --person nazwisko - wiadomosci od jakiej osoby mają być wyświetlone. Trzeba podac nazwisko osoby (lub jego część)
                    - --pages pages - z jakich stron mają być pobrane wiadomosci. Format to PierwszStrona-OstatniaStrona, np 2-4 pobierze strony 2, 3 i 4

    exit - zamyka program

    cls - czyści ekran

    login - loguje do librusa
        Użycie:
            login <arguments>
                arguments:
                    - --login login - login do librusa
                    - --password password - hasło do librusa

    logout - wylogowuje

    rmes - odczytuje wiadomość zapisaną w pamięci programu. Wcześniej trzeba użyć komendy gmes która zapisuje wiadomosci w pamięci
        Użycie:
            rmes indeks
                options:
                    - indeks - indeks wiadomosci którą chcemy odczytać.

    help - wyświetla tą wiadomosć

    grades - pobiera i wyświetla oceny ucznia
        Użycie:
            login <arguments>
                arguments:
                    - --detailed - oceny będą wyświetlone ze szczegółami

    abs - wyświetla nieobecności ucznia
"""

    print(message)



from librus_terminal.librus_session import LibrusSession
import librus_scraper


def _print_grade_detailed(grade: librus_scraper.grades.Grade, indentation: str, index: int) -> None:
    print(
        indentation,
        str(index).rjust(2),
        "  " if grade.to_max_points else " Z",
        "  " if grade.mean else "I ",
        "{}/{} ".format(*grade.points),
        f"{round((grade.points[0] / grade.points[1]) * 100, 2)}% " if grade.points[1] else " ",
        grade.category,
        sep=""
    )


def _print_semester_grades(semester: librus_scraper.grades.Semester, semester_index: int, detailed: bool = False) -> None:
    print(f"\tSemester {semester_index}:", end=" " if not detailed else "\n")

    if not detailed:
        grades = ["{}/{}".format(*grade.points) for grade in semester.grades]

        print(*grades)
        return

    for index, grade in enumerate(semester.grades, 1):
        _print_grade_detailed(grade, indentation="\t\t", index=index)


def grades_command(session: LibrusSession, *, detailed: bool = False) -> None:
    grades = librus_scraper.grades.get_grades_detailed(session.cookies)

    for subject in grades:
        print(subject.name, f"{subject.points_sum}/{subject.points_max} {round(subject.percentage, 2)}%,", "Oceny:")

        for semester_index, semester in enumerate(subject.semesters, 1):
            _print_semester_grades(semester, semester_index, detailed=detailed)



from .librus_session import LibrusSession
from .command_parsing import parse_command
from .exceptions import ParsingException, TerminalAuthorizationException
import re

from . import messages_commands
from . import other_commands
from . import authorization_commands
from . import help_command
from . import grades_commands


class LibrusTerminal():

    enable_color = True
    commands = {
        "gmes": messages_commands.get_messages_command,
        "exit": other_commands.exit_command,
        "cls": other_commands.clear_command,
        "clear": other_commands.clear_command,
        "login": authorization_commands.login_command,
        "logout": authorization_commands.logout_command,
        "rmes": messages_commands.read_message_command,
        "help": help_command.help_command,
        "abs": other_commands.absences_command,
        "grades": grades_commands.grades_command
    }

    def __init__(self, session: LibrusSession) -> None:
        self.session: LibrusSession = session

    @property
    def _input_message(self) -> str:
        logged_in = True

        try:
            self.session.check_if_expired()

        except TerminalAuthorizationException as exception:
            logged_in = False

        attrs = ("logged in", "BLUE") if logged_in else ("not logged in", "RED")

        message =\
            self._color("[Librus (", "GREEN") +\
            self._color(*attrs) +\
            self._color(")] >> ", "GREEN")

        return message

    @classmethod
    def _color(cls, text, col):

        if not cls.enable_color:
            return text

        colors = {
            "BLUE": '\033[94m',
            "GREEN": '\033[92m',
            "YELLOW": '\033[93m',
            "RED": '\033[91m',
            "ENDC": '\033[0m',
        }

        col = colors.get(col, "")

        return col + text + (colors["ENDC"] if col else "")

    def run_terminal(self) -> None:
        while True:
            try:
                command = input(self._input_message)
            except KeyboardInterrupt:
                exit()

            if not command:
                continue

            try:
                options, arguments = parse_command(command)
            except ParsingException as exception:
                print(str(exception), *exception.errors)
                continue

            except ValueError as exception:
                print("niepoprawnie sformatowana komenda", str(exception))
                continue

            command, options = options[0], options[1:]

            if command not in self.commands:
                print("nie ma takiej komendy:", command)
                continue

            try:
                result = self.commands[command](self.session, *options, **arguments)

            except TerminalAuthorizationException as exception:
                print(exception.message_for_user)

            except TypeError as exception:
                error_message = str(exception)

                if "got an unexpected keyword argument" in error_message:
                    print("nieznany argument:", "--" + error_message.split("'")[1])

                elif "required positional argument" in error_message:
                    print("nie podano wymaganych opcji")

                elif (match := re.match(r".+(\d+) positional argument but (\d+).+", error_message)) is not None:
                    print("ta komenda wymaga {} opcji, a podano {}".format(
                        int(match.groups()[0]) - 1,
                        int(match.groups()[1]) - 1
                    ))
                else:
                    raise exception

            except KeyboardInterrupt as exception:
                continue



class ParsingException(Exception):
    def __init__(self, message, errors) -> None:
        super().__init__(message)

        self.errors = errors


class TerminalAuthorizationException(Exception):
    def __init__(self, message, message_for_user) -> None:
        super().__init__(message)

        self.message_for_user = message_for_user



from .librus_session import LibrusSession
import librus_scraper
import re


def _ljust_with_dots(string: str, length: int, fillchar: str = " ", inc_space: bool = True) -> int:
    if len(string) <= length:
        return string.ljust(length, fillchar) + " " * inc_space

    else:
        return string[:length - 3] + "..." + " " * inc_space


def print_message(index: int, mes: dict) -> None:

    string = \
        _ljust_with_dots(str(index), 4, inc_space=0) +\
        (("P" * mes["files"]) or " ") + (("N " * mes["new"]) or "  ") +\
        mes["data"].split(" ")[0] + " " +\
        _ljust_with_dots(mes["nadawca"].split("(")[0], 20) +\
        _ljust_with_dots(mes["temat"], 60)

    print(string)


def print_messages(messages: list[dict], pages: list[int] = None) -> None:
    for index, message in enumerate(messages):
        print_message(index + 1, message)

    if pages is not None:
        print("{}-{}/{}".format(*pages))


def get_messages_command(
    session: LibrusSession, *, source: str = "current", person: str = "-", pages: str = None
) -> int:

    if source == "current":
        archive = False

    elif source == "archive":
        archive = True

    elif source == "memory":
        if session.messages is None:
            print("nie ma wiadomości zapisanych w pamięci")
            return 1

        print_messages(session.messages, pages=session.messages_pages)

        return 0

    else:
        print("nieznane źródło wiadomości")
        return 1

    if person != "-":
        people = librus_scraper.messages.get_senders_id(session.cookies, archive=archive)

        for person_id, person_name in people[1:]:
            if person_name.lower().startswith(person.lower()):
                person = person_id
                break

        else:
            print("nie ma takiej osoby")
            return 1

    if pages is None:
        pages = "1-1"

    match = re.match(r"^(\d{0,})-(\d{0,})$", pages)

    if match is None:
        print("niepoprawny format stron (np. 1-10)")
        return 1

    page_range = [
        int(match.groups()[0]) if match.groups()[0] else None,
        int(match.groups()[1]) if match.groups()[1] else None,
    ]

    pagination = [
        (page_range[0] - 1) if page_range[0] is not None else 0
    ]

    pagination.append(pagination[0] + 1)

    messages: list[dict] = []

    pages: list[int] = [
        page_range[0] if page_range[0] is not None else 1, 0, 0
    ]

    while pagination[0] < pagination[1]:
        if page_range[1] is not None and pagination[0] >= page_range[1]:
            break

        result = librus_scraper.messages.get_messages(
            cookies=session.cookies,
            archive=archive,
            person=person,
            page=str(pagination[0])
        )

        pagination = result["pagination"]

        pages = [
            min(pagination[0], pages[0]),
            max(pages[1], pagination[0]),
            pagination[1]
        ]

        messages += result["messages"]

        if page_range[0] is not None and pagination[1] < page_range[0]:
            break

    if not messages:
        print("żadne wiadomości nie spełniają tych warunków")
        return 1

    session.messages = messages
    session.messages_pages = pages

    print_messages(session.messages, pages=session.messages_pages)


def read_message_command(session: LibrusSession, index: str) -> int:
    if session.messages is None:
        print("nie ma wiadomości w pamięci")
        return 1

    match = re.match(r"^\d+$", index)

    if match is None:
        print(index, "nie jest liczbą")
        return 1

    index = int(index) - 1

    if index not in range(len(session.messages)):
        print("nie ma wiadomości o takim indeksie")
        return 1

    message = librus_scraper.read_message(session.cookies, session.messages[index]["href"])

    print("Nadawca:", message["nadawca"])
    print("Data:", message["data"])
    print("Temat:", message["temat"], "\n")
    print(message["tresc"])



from .librus_session import LibrusSession
import librus_scraper
import os


def exit_command(session: LibrusSession) -> None:
    exit()


def clear_command(session: LibrusSession) -> None:
    os.system("cls")


def absences_command(session: LibrusSession) -> None:
    cookies = session.cookies

    absences_generator = librus_scraper.get_attendence(cookies)

    for absence in absences_generator:
        message = ""

        for entry in absence["entries"]:
            if entry["Rodzaj"] != "nieobecność":
                continue

            message += (
                f"\t{entry['Godzina lekcyjna']} - {entry['Lekcja']}\n"
            )

        if not message:
            continue

        print(absence["date"], message, sep="\n")



from shlex import split as split_command
from .exceptions import ParsingException


def parse_command(command: str) -> tuple[list, dict]:
    tokens = split_command(command)

    options, argument_tokens = [], []

    for index, token in enumerate(tokens):
        if token.startswith("--"):
            argument_tokens = tokens[index:]
            break

        options.append(token)

    prev_token, arguments = None, {}

    for token in argument_tokens:
        if token.startswith("--"):
            if prev_token is not None:
                arguments[prev_token.strip("--")] = True

            prev_token = token
            continue

        if prev_token is None:
            raise ParsingException("Options must be given before arguments:", [token])

        arguments[prev_token.strip("--")] = token

        prev_token = None

    if prev_token is not None:
        arguments[prev_token.strip("--")] = True

    return options, arguments



from .librus_session import LibrusSession
from .exceptions import TerminalAuthorizationException
from librus_scraper.authorization import AuthorizationException as LibrusAuthException
import json


def logout_command(session: LibrusSession) -> None:
    session._cookies = None
    session._cookie_session_expires = 0
    session._csrf_token = None


def login_command(session: LibrusSession, *, login: str = None, password: str = None) -> None:

    while not login:
        login = input("[login] >> ")

    while not password:
        password = input("[password] >> ")

    try:
        session.login(login, password)

    except LibrusAuthException:
        raise TerminalAuthorizationException("invalid login or password", "nieprawidłowe hasło lub login")



from librus_terminal.command_parsing import parse_command
import librus_scraper
from .exceptions import TerminalAuthorizationException
import time


class LibrusSession():

    minimum_left = 10

    def __init__(self) -> None:
        self._cookies = None
        self._csrf_token = None

        self.messages = None
        self.messages_pages = None

        self._cookie_session_expires = 0

    def login(self, login: str, password: str) -> None:
        cookies = librus_scraper.authorization._get_cookies(login, password)

        expires = 0

        for cookie in cookies:
            if cookie.name != "oauth_token":
                continue

            expires = cookie.expires
            break

        self._cookies = dict(cookies)

        self._cookie_session_expires = expires

        self._csrf_token = librus_scraper.get_csrf_token(self.cookies)

    def check_if_expired(self) -> None:
        time_left = self._cookie_session_expires - round(time.time())

        if time_left < self.minimum_left:
            raise TerminalAuthorizationException(
                "cookies expired", "sesja wygasła, musisz się ponownie zalogować"
            )

    @property
    def cookies(self) -> dict:
        if self._cookies is not None:
            self.check_if_expired()
            return self._cookies

        raise TerminalAuthorizationException(
            "no cookies for this session", "musisz się zalogowac"
        )

    @property
    def csrf_token(self) -> str:
        if self._csrf_token is not None:
            self.check_if_expired()
            return self._csrf_token

        raise TerminalAuthorizationException(
            "no csrf_token for this session", "musisz się zalogowac"
        )



from setuptools import setup

setup(
    name="librus_terminal",
    version="0.1.1",
    author="Piotr Ginał",
    packages=["librus_terminal"],
    install_requires=[
        "librus_scraper"
    ],
    entry_points={
        "console_scripts": [
            "librus_terminal=librus_terminal.__main__:main"
        ]
    }
)
